import { Context } from 'skypager-registry'
import Helper from '../helper'

export class DocumentType extends Helper {
  static isCacheable = true

  static cacheable(setting = true) {
    return this.isCacheable = !!setting
  }

  static createRegistry = () => new Context('documentTypes', {
    context: require.context('../document-types', false, /.js$/),
  })

  static attach(project, options = {}) {
    return Helper.attach(project, DocumentType, {
      registryProp: 'documentTypes',
      lookupProp: 'documentType',
      registry: options.registry || DocumentType.createRegistry(),
      ...options,
    })
  }

  get matchingDocuments() {
    return this.loadMatchingDocuments({ready: false})
  }

  loadMatchingDocuments(options = {}) {
    const { 
      ready = true,
      pattern = '*'
    } = options

    return this.project
      .route(pattern)
      .filter(r => this.testDocument(r.doc))
      .map(({doc}) =>  ready ? doc.ready : doc)
  }

  testDocument(doc) {
    return this.provider.testDocument(doc)
  }

  decorate(doc) {
    return this.provider.decorate(doc)
  }
}

export default DocumentType
