import remark from 'remark'
import commentBlocks from 'remark-comment-blocks'
import commentConfig from 'remark-comment-config'
import remarkHtml from 'remark-html'
import slug from 'remark-slug'
import squeeze from 'remark-squeeze-paragraphs'
import annotations from 'remark-yaml-annotations'
import stringify from 'remark-stringify'
import normalize from 'remark-normalize-headings'
import githubReferences from 'remark-github'
import toc from 'remark-toc'
import autolink from 'remark-autolink-headings'
import contributors from 'remark-contributors'
import license from 'remark-license'
import stripBadges from 'remark-strip-badges'
import usage from 'remark-usage'
import validateLinks from 'remark-validate-links'
import parse from 'remark-parse'

/**
 * Run a remark parser profile with optional add-ons. Available profiles
 * can be found @see profiles and addOns @see addOns
 *
 * @param {Object} options - the options object
 * @param {String} options.profile - which parser profile to use?
 * @param {Array} options.addOns - which addons to include?
 */
export default (options = {}, context = {}) => {
  const { profile = 'parser', addOns = [] } = options
  const unified = profiles[profile] || parser

  return addOns.reduce((memo,addon) => (
    addons[addOn] ? memo.use(addons[addOn]) : memo
  ), unified(options))
}

export const applyTo = (input, options) => {
  const { method = 'parse', parser = 'github' } = options
  return profiles[parser](options)[method](input)
}

export const standard = (options = {}) =>
  remark()
    .use(commentConfig)
    .use(normalize)
    .use(squeeze)
    .use(slug)
    .use(annotations)
    .use(autolink)

export const comments = (options = {}) =>
  standard(options).use(commentBlocks)

export const parser = (options = {}) =>
  standard(options).use(parse)

export const html = (options = {}) =>
  standard(options).use(remarkHtml)

export const stringifier = (options = {}) =>
  standard(options).use(stringify)

export const github = (options = {}) =>
  standard(options)
    .use(githubReferences)
    .use(autolink)

export const profiles = {
  parser,
  standard,
  comments,
  github,
  html,
  stringifier,
}

export const addons = {
  contributors,
  license,
  stripBadges,
  usage,
  validateLinks,
  toc
}
